import { Vector3 } from '@babylonjs/core/Maths/math';
import { TransformNode } from '@babylonjs/core/Meshes/transformNode';
import { Nullable } from '@babylonjs/core/types';
import { SphereCollider } from './sphere-collider';
/**
 * Verlet Spring Bone Logic
 */
export declare class VRMSpringBoneLogic {
    readonly radius: number;
    readonly transform: TransformNode;
    /**
     * Cloned initial local rotation
     */
    private readonly localRotation;
    /**
     * Reference of parent rotation
     */
    private readonly parentRotation;
    private readonly boneAxis;
    private readonly boneLength;
    private currentTail;
    private prevTail;
    /**
     * @param center Center reference of TransformNode
     * @param radius Collision Radius
     * @param transform Base TransformNode
     * @param localChildPosition
     */
    constructor(center: Nullable<TransformNode>, radius: number, transform: TransformNode, localChildPosition: Vector3);
    /**
     * Update Tail position
     *
     * @param center Center reference of TransformNode
     * @param stiffnessForce Current frame stiffness
     * @param dragForce Current frame drag force
     * @param external Current frame external force
     * @param colliders Current frame colliders
     */
    update(center: Nullable<TransformNode>, stiffnessForce: number, dragForce: number, external: Vector3, colliders: SphereCollider[]): void;
    private getCenterTranslatedWorldPos;
    private getCenterTranslatedPos;
    /**
     * 次のテールの位置情報から回転情報を生成する
     *
     * @see https://stackoverflow.com/questions/51549366/what-is-the-math-behind-fromtorotation-unity3d
     */
    private transformToRotation;
    /**
     * 衝突判定を行う
     * @param colliders SphereColliders
     * @param nextTail NextTail
     */
    private collide;
}
